package ru.dbotthepony.kstarbound.defs.actor.behavior

import com.google.gson.TypeAdapter
import com.google.gson.annotations.JsonAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.gson.get
import ru.dbotthepony.kstarbound.json.popObject
import ru.dbotthepony.kstarbound.util.asStringOrNull
import ru.dbotthepony.kstarbound.util.valueOf
import ru.dbotthepony.kstarbound.world.entities.behavior.NodeParameterType

@JsonAdapter(NodeOutput.Adapter::class)
data class NodeOutput(val type: NodeParameterType, val key: String? = null, val ephemeral: Boolean = false) {
	class Adapter : TypeAdapter<NodeOutput>() {
		override fun write(out: JsonWriter, value: NodeOutput) {
			out.beginObject()
			out.name("type")
			out.value(value.type.jsonName)

			if (value.key != null) {
				out.name("key")
				out.value(value.key)
			}

			out.name("ephemeral")
			out.value(value.ephemeral)

			out.endObject()
		}

		override fun read(`in`: JsonReader): NodeOutput {
			val json = `in`.popObject()

			return NodeOutput(
				NodeParameterType.entries.valueOf(json["type"].asString),
				json["key"]?.asStringOrNull,
				json.get("ephemeral", false)
			)
		}
	}
}
